@@ -1,5 +1,8 @@
 #' @template function_online
 #'
+#' @description
+#' `r lifecycle::badge("stable")`
+#'
 #' @template param_y
 #' @param experts An array of predictions with dimension T x D x P x K
 #' (Observations x Variables x Quantiles x Experts) or T x D x K or T x P x K.
@@ -41,6 +44,15 @@
 #' @template param_parametergrids
 #' @template param_forget_past_performance
 #'
+#' @param save_past_performance Whether or not the past performance w.r.t to the
+#' considered parameter grid should be reported or not. Defaults to `FALSE` to
+#' save memory. Setting it to `TRUE` can be memory intensive depending on the
+#' data and the considered grid.
+#' @param save_predictions_grid Whether or not all predictions w.r.t to the
+#' considered parameter grid should be reported or not. Defaults to `FALSE`.
+#' Setting it to `TRUE` can be memory intensive depending on the data
+#'  and the considered grid.
+#'
 #' @template param_allow_quantile_crossing
 #'
 #' @param init A named list containing "init_weights": Array of dimension
@@ -90,10 +102,12 @@
 #' # Update will update the models weights etc if you provide new realizations
 #' model <- update(model, new_y = new_y, new_experts = new_experts)
 #'
-#' # Predict will expand \code{model$predictions} by default
+#' # Predict will expand `model$predictions` by default
 #' model <- predict(model, new_experts = new_experts, update_model = TRUE)
 #' }
+#'
 #' @importFrom abind adrop asub
+#'
 #' @export
 online <- function(y, experts, tau,
                    lead_time = 0,
@@ -146,10 +160,13 @@ online <- function(y, experts, tau,
                    parametergrids = list(
                        general = NULL,
                        b_smooth_pr = NULL,
-                       p_smooth_pr = NULL, b_smooth_mv = NULL,
+                       p_smooth_pr = NULL,
+                       b_smooth_mv = NULL,
                        p_smooth_mv = NULL
                    ),
                    forget_past_performance = 0,
+                   save_past_performance = FALSE,
+                   save_predictions_grid = FALSE,
                    allow_quantile_crossing = FALSE,
                    init = NULL,
                    loss = NULL,
@@ -159,6 +176,8 @@ online <- function(y, experts, tau,
     model_instance$trace <- trace
     model_instance$tau <- tau
     model_instance$forget_past_performance <- forget_past_performance
+    model_instance$save_past_performance <- save_past_performance
+    model_instance$save_predictions_grid <- save_predictions_grid
 
 
     edim <- dim(experts)
@@ -313,7 +332,7 @@ online <- function(y, experts, tau,
         nonc = val_or_def(p_smooth_pr$nonc, 0),
         tailw = val_or_def(p_smooth_pr$tailweight, 1),
         deg = val_or_def(p_smooth_pr$deg, 1),
-        diff = val_or_def(p_smooth_pr$diff, 1.5),
+        ndiff = val_or_def(p_smooth_pr$ndiff, 1.5),
         lambda = val_or_def(p_smooth_pr$lambda, -Inf)
     )
     pars_hat_pr_n <- prod(sapply(pars_hat_pr, length))
@@ -325,27 +344,28 @@ online <- function(y, experts, tau,
         nonc = val_or_def(p_smooth_mv$nonc, 0),
         tailw = val_or_def(p_smooth_mv$tailweight, 1),
         deg = val_or_def(p_smooth_mv$deg, 1),
-        diff = val_or_def(p_smooth_mv$diff, 1.5),
+        ndiff = val_or_def(p_smooth_mv$ndiff, 1.5),
         lambda = val_or_def(p_smooth_mv$lambda, -Inf)
     )
     pars_hat_mv_n <- prod(sapply(pars_hat_mv, length))
 
     if (is.null(parametergrids$general)) {
-        parametergrid <- expand_grid_sample(list(
-            forget_regret = forget_regret,
-            soft_threshold = soft_threshold,
-            hard_threshold = hard_threshold,
-            fixed_share = fixed_share,
-            basis_pr_idx = seq_len(pars_basis_pr_n),
-            basis_mv_idx = seq_len(pars_basis_mv_n),
-            hat_pr_idx = seq_len(pars_hat_pr_n),
-            hat_mv_idx = seq_len(pars_hat_mv_n),
-            gamma = gamma,
-            loss_share = loss_share,
-            regret_share = regret_share
-        ),
-        n = parametergrid_max_combinations,
-        verbose = TRUE
+        parametergrid <- expand_grid_sample(
+            list(
+                forget_regret = forget_regret,
+                soft_threshold = soft_threshold,
+                hard_threshold = hard_threshold,
+                fixed_share = fixed_share,
+                basis_pr_idx = seq_len(pars_basis_pr_n),
+                basis_mv_idx = seq_len(pars_basis_mv_n),
+                hat_pr_idx = seq_len(pars_hat_pr_n),
+                hat_mv_idx = seq_len(pars_hat_mv_n),
+                gamma = gamma,
+                loss_share = loss_share,
+                regret_share = regret_share
+            ),
+            n = parametergrid_max_combinations,
+            verbose = TRUE
         )
     } else {
         parametergrid <- parametergrids$general
@@ -391,7 +411,7 @@ online <- function(y, experts, tau,
         nonc = val_or_def(p_smooth_pr$nonc, 0),
         tailw = val_or_def(p_smooth_pr$tailweight, 1),
         deg = val_or_def(p_smooth_pr$deg, 1),
-        diff = val_or_def(p_smooth_pr$diff, 1.5),
+        ndiff = val_or_def(p_smooth_pr$ndiff, 1.5),
         lambda = val_or_def(p_smooth_pr$lambda, -Inf),
         idx = sort(unique(parametergrid[, "hat_pr_idx"])),
         params = parametergrids$p_smooth_pr
@@ -407,7 +427,7 @@ online <- function(y, experts, tau,
         nonc = val_or_def(p_smooth_mv$nonc, 0),
         tailw = val_or_def(p_smooth_mv$tailweight, 1),
         deg = val_or_def(p_smooth_mv$deg, 1),
-        diff = val_or_def(p_smooth_mv$diff, 1.5),
+        ndiff = val_or_def(p_smooth_mv$ndiff, 1.5),
         lambda = val_or_def(p_smooth_mv$lambda, -Inf),
         idx = sort(unique(parametergrid[, "hat_mv_idx"])),
         params = parametergrids$p_smooth_mv
