@@ -36,14 +36,16 @@ boa_smooth <- online(
 
 expect_true(all(dim(boa_smooth$past_performance) == c(1000, 1, 99, 8)))
 
-expect_true(
-    all(round(boa_smooth$past_performance[T, 1, 50, ], 7) ==
-        c(
-            0.2728130, 0.3878642, 0.3796495, 0.3793674,
-            0.3309897, 0.3310782, 0.2906660, 0.2907555
-        ))
+expect_equal(
+    as.numeric(boa_smooth$past_performance[T, 1, 50, ]),
+    c(
+        0.2728130, 0.3878642, 0.3796495, 0.3793674,
+        0.3309897, 0.3310782, 0.2906660, 0.2907555
+    ),
+    tolerance = 0.0000001
 )
 
+
 expect_true(boa_smooth$specification$parameters$save_past_performance)
 
 # The default of save_past_performance is expected to be FALSE
